@@ -91,7 +91,7 @@
 
 (define (esc->ch ch)
   (case ch ((#\nul) #\0) ((#\bel) #\a) ((#\bs) #\b) ((#\ht) #\t)
-        ((#\nl) #\n) ((#\vt) #\v) ((#\np) #\f) ((#\cr) #\r)))
+        ((#\nl) #\n) ((#\vt) #\v) ((#\np) #\f) ((#\cr) #\r) (else ch)))
    
 ;; @deffn {Procedure} scmstr->c str
 ;; to be documented
@@ -101,10 +101,10 @@
    (string-fold-right
     (lambda (ch chl)
       (cond
+       ((memq ch '(#\\ #\" #\nul #\bel #\bs #\ht #\nl #\vt #\np #\cr))
+        (cons* #\\ (esc->ch ch) chl))
        ((char-set-contains? char-set:printing ch) (cons ch chl))
        ((char=? ch #\space) (cons #\space chl))
-       ((memq ch '(#\nul #\bel #\bs #\ht #\nl #\vt #\np #\cr))
-        (cons* #\\ (esc->ch ch) chl))
        (else (char->hex-list ch chl))))
     '() str)))
     
@@ -334,6 +334,7 @@
 
       ;; TODO: check protection
       ((comma-expr . ,expr-list)
+       (sf "(")
        (pair-for-each
         (lambda (pair)
           (cond
@@ -343,7 +344,8 @@
                 (ppx (car pair)))
             (sf ", "))
            (else (ppx (car pair)))))
-        expr-list))
+        expr-list)
+       (sf ")"))
 
       ((stmt-expr (block-item-list . ,items))
        (sf "({\n") (push-il) (for-each ppx items) (pop-il) (sf "})"))
